from __future__ import unicode_literals

from django.db import models

from guardian.shortcuts import get_objects_for_user


class MessageQueryset(models.QuerySet):
    # TODO: Remove dependency of guardian

    def _get_messages(self, user, folder, obj, **kwargs):
        if obj:
            kwargs.update({'object_id': obj.pk})
        qs = self.filter(folder=folder, **kwargs)
        return get_objects_for_user(
            user, 'view_message', qs, use_groups=False, with_superuser=False)

    def inbox(self, user, namespace, obj=None):
        return self._get_messages(
            user, self.model.Folder.RECEIVED, obj, recipient__user=user,
            recipient__namespace=namespace)

    def sentbox(self, user, namespace, obj=None):
        return self._get_messages(
            user, self.model.Folder.SENT, obj, sender__user=user,
            sender__namespace=namespace)

    def archivedbox(self, user, namespace, obj=None):
        return self._get_messages(
            user, self.model.Folder.ARCHIVED, obj, recipient__user=user,
            recipient__namespace=namespace)

    def unread(self):
        return self.filter(read_at=None)


class MessageContactManager(models.Manager):

    def get_or_create_contact(self, user, namespace, name=None, email=None):
        """ """
        if user.is_authenticated():
            contact, _ = self.update_or_create(
                user_id=user.pk, namespace=namespace, name=name,
                defaults={'email': email})
        elif email:
            contact, _ = self.get_or_create(email=email, namespace=namespace,
                                            name=name)
        else:
            raise ValueError("No email provided for anonymous user")
        return contact

    def get_or_create_contacts(self, namespaces):
        """ """
        to_create = []
        missing_user_ids = []
        # calculate missing contacts for users
        for namespace, user_ids in namespaces.items():
            created_user_ids = self.filter(
                user_id__in=user_ids, namespace=namespace)\
                .values_list('user_id', flat=True)
            missing_user_ids += list(set(user_ids) - set(created_user_ids))
        # add to bulk create new contacts
        for missing_user_id in missing_user_ids:
            to_create.append(self.model(
                user_id=missing_user_id,
                namespace=namespace
            ))
        # create
        if to_create:
            self.bulk_create(to_create)
        # return all contacts created
        return self.filter(user_id__in=user_ids, namespace=namespace)
