
NAME_SEPARATOR = ':'


class BaseRecipient(object):
    """ """
    model = None
    prefix = ''
    namespace = None
    label_prefix = None

    def __init__(self, user, query='', group_object_id=None):
        self.user = user
        self.query = query
        self.group_object_id = group_object_id

    def get_object(self, name):
        (prefix, pk) = name.split(NAME_SEPARATOR)
        if prefix == self.prefix:
            return self.get_queryset().filter(pk=pk).first()
        return None

    def get_queryset(self):
        return self.model._default_manager.all()

    def get_label(self, obj):
        return obj.__str__()

    def check_permission(self):
        return True

    def get_user_ids(self, obj):
        raise NotImplementedError

    def get_group_object(self, obj):
        return None

    def get_recipients(self, limit=None):
        qs = self.get_queryset()
        items = []
        for item in qs[:limit]:
            items.append({
                'id': self.gen_recipient_id(item),
                'text': self.format_label(item)
            })
        return items

    def gen_recipient_id(self, obj):
        return "{}{}{}".format(self.prefix, NAME_SEPARATOR, obj.pk)

    def format_label(self, obj):
        prefix = (self.label_prefix + " ") if self.label_prefix else ""
        return "{}{}".format(prefix, self.get_label(obj))
