from django.utils.module_loading import autodiscover_modules


class RecipientRegister(object):
    """ """
    _registry = []

    def add(self, recipient):
        self._registry.append(recipient)

    def search(self, user, query='', group_object_id=None, limit=None):
        recipients = []
        for recipient_class in self._registry:
            recipient = recipient_class(user, query, group_object_id)
            if recipient.check_permission():
                recipients += recipient.get_recipients(limit=limit)
        # TODO: order by label
        return recipients

    def get_by_namespace(self, user, recipient_ids, group_object_id=None):
        users = {}
        for recipient_class in self._registry:
            recipient = recipient_class(user, group_object_id=group_object_id)
            for recipient_id in recipient_ids:
                obj = recipient.get_object(recipient_id)
                recipient_users = recipient.get_user_ids(obj)
                if recipient_users:
                    users.setdefault(recipient.namespace, [])
                    users[recipient.namespace] += recipient_users
        return users


recipient_register = RecipientRegister()


def autodiscover():
    """autodiscover and import conditions.py files to feed register
    """
    autodiscover_modules('recipients', register_to=recipient_register)
