from __future__ import unicode_literals

from django.apps import apps

from guardian.shortcuts import assign_perm
from evotecore.utils import list_portions
from evote.celery import app

from .mailings import MessageMail


@app.task(ignore_result=True)
def send_messages(to_send_messageid, recipients_ids):
    Message = apps.get_model("evotemessages.Message")  # NOQA
    to_send = Message.objects.get(pk=to_send_messageid)
    to_send.mark_as_read()
    messages_to_send = []
    for recipient_id in recipients_ids:
        messages_to_send.append(Message(
            sender_id=to_send.sender_id,
            subject=to_send.subject,
            recipient_id=recipient_id,
            body=to_send.body,
            parent=to_send,
            folder=to_send.Folder.RECEIVED,
            state=to_send.State.NEW,
            content_type_id=to_send.content_type_id,
            object_id=to_send.object_id
        ))
    for portion in list_portions(messages_to_send, 3000):
        Message.objects.bulk_create(portion)
    update_messages_permissions.si(to_send.pk).delay()
    return len(messages_to_send)


@app.task(ignore_result=True)
def update_messages_permissions(sended_messageid):
    Message = apps.get_model("evotemessages.Message")  # NOQA
    # add view message permission to sender user if defined
    sended_msg = Message.objects.get(pk=sended_messageid)
    if sended_msg.sender.is_anonymous:
        MessageMail(sended_msg).send(sended_msg.sender)
    else:
        assign_perm("evotemessages.view_message", sended_msg.sender.user,
                    sended_msg)
    # add view message permission to all recipients
    messages = Message.objects.filter(parent_id=sended_messageid)
    for message in messages:
        assign_perm("evotemessages.view_message", message.recipient.user,
                    message)
        MessageMail(message).send(message.recipient)
