from __future__ import unicode_literals

from django.utils.translation import ugettext as _
from django.db.models import Q

from evotemessages.decorators import recipients
from evotemessages.recipients import BaseRecipient

from .models import Category, Election


@recipients
class CategoryRecipient(BaseRecipient):
    """
    Group of voters of election category
    """
    model = Category
    prefix = "election_category"
    namespace = "voters"
    label_prefix = _("Category:")

    def get_queryset(self):
        if not self.group_object_id:  # if no group provided, return empty
            return self.model.objects.none()
        qs = super(CategoryRecipient, self).get_queryset()
        return qs.select_related('election')\
            .filter(Q(designation__icontains=self.query) |
                    Q(election__identifier__icontains=self.query),
                    election__electoral_process_id=self.group_object_id)\
            .order_by()

    def get_user_ids(self, obj):
        if obj:
            return obj.election_voters.active()\
                .values_list("user_id", flat=True).order_by()
        return []

    def get_label(self, obj):
        return "{} ({})".format(obj.designation, obj.election)

    def check_permission(self):
        return getattr(self.user, "is_electoral_member", False)


@recipients
class ElectionRecipient(BaseRecipient):
    """"""
    model = Election
    prefix = "election"
    namespace = "voters"
    label_prefix = _("Election:")

    def get_queryset(self):
        if not self.group_object_id:  # if no group provided, return empty
            return self.model.objects.none()
        qs = super(ElectionRecipient, self).get_queryset()
        return qs.filter(identifier__icontains=self.query,
                         electoral_process_id=self.group_object_id).order_by()

    def get_user_ids(self, obj):
        if obj:
            return obj.election_voters.active()\
                .values_list("user_id", flat=True).order_by()
        return []

    def check_permission(self):
        return getattr(self.user, "is_electoral_member", False)
