from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _
from django.db.models import Q

from evotemessages.decorators import recipients
from evotemessages.recipients import BaseRecipient
from guardian.shortcuts import get_objects_for_user

from .models import ElectoralProcess, ElectoralMember


@recipients
class ElectoralCommissionRecipient(BaseRecipient):
    """ """
    model = ElectoralProcess
    prefix = "electoral_commission"
    namespace = "management"
    label_prefix = _("Electoral commission:")

    def get_queryset(self):
        qs = super(ElectoralCommissionRecipient, self).get_queryset()
        extra = {}
        if self.group_object_id:
            extra.update({'pk': self.group_object_id})
        else:
            qs = get_objects_for_user(
                self.user, 'view_electoralprocess', qs, use_groups=False,
                with_superuser=False).published()
        return qs.filter(identifier__icontains=self.query, **extra).order_by()

    def get_user_ids(self, obj):
        if obj:
            return obj.electoral_members.values_list("user_id", flat=True)\
                .order_by()
        return []

    def get_group_object(self, obj):
        return obj

    def check_permission(self):
        is_member = getattr(self.user, "is_electoral_member", False)
        is_voter = getattr(self.user, "is_election_voter", False)
        return bool(is_member or is_voter)


@recipients
class ElectoralMemberRecipient(BaseRecipient):
    """ """
    model = ElectoralMember
    prefix = "electoral_member"
    namespace = "management"
    label_prefix = _("Electoral member:")

    def get_queryset(self):
        if not self.group_object_id:  # if no group provided, return empty
            return self.model.objects.none()
        qs = super(ElectoralMemberRecipient, self).get_queryset()
        return qs.select_related('user')\
            .filter(Q(designation__icontains=self.query) |
                    Q(user__email__icontains=self.query) |
                    Q(user__notice_email__icontains=self.query),
                    electoral_process_id=self.group_object_id)\
            .order_by()

    def get_user_ids(self, obj):
        if obj:
            return [obj.user_id]
        return []

    def check_permission(self):
        return getattr(self.user, "is_electoral_member", False)
