from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _
from django.db.models import Q

from evotemessages.decorators import recipients
from evotemessages.recipients import BaseRecipient

from .models import ElectionVoter


@recipients
class ElectionVoterRecipient(BaseRecipient):
    """ """
    model = ElectionVoter
    prefix = 'election_voter'
    namespace = "voters"
    label_prefix = _('Election voter:')

    def get_queryset(self):
        if not self.group_object_id:  # if no group provided, return empty
            return self.model.objects.none()
        qs = super(ElectionVoterRecipient, self).get_queryset()
        query = qs.select_related('user', 'election')\
            .filter(Q(name__icontains=self.query) |
                    Q(personnel_number__icontains=self.query) |
                    Q(user__email__icontains=self.query) |
                    Q(user__notice_email__icontains=self.query),
                    election__electoral_process_id=self.group_object_id)\
            .order_by()
        return query

    def get_user_ids(self, obj):
        if obj:
            return [obj.user_id]
        return []

    def check_permission(self):
        return getattr(self.user, 'is_electoral_member', False)
